;------------------------------------------------------------------------------
;  FILENAME:   Counter16_1.asm
;   VERSION:   Rev B, 2002 Mar 30
;------------------------------------------------------------------------------
;  DESCRIPTION:
;     Counter16_1 Counter16 User Module API.
;
;------------------------------------------------------------------------------
;	Copyright (c) Cypress MicroSystems 2000-2002. All Rights Reserved.
;------------------------------------------------------------------------------

;-----------------------------------------------
; include instance specific register definitions
;-----------------------------------------------
include "m8c.inc"
include "Counter16_1.inc"

   area text (ROM, REL)

;-------------------------------------------------------------------
;  Declare the functions global for both assembler and C compiler.
;
;  Note that there are two names for each API. First name is 
;  assembler reference. Name with underscore is name refence for
;  C compiler.  Calling function in C source code does not require 
;  the underscore.
;-------------------------------------------------------------------
export    Counter16_1_EnableInt 
export   _Counter16_1_EnableInt
export    Counter16_1_DisableInt 
export   _Counter16_1_DisableInt 
export    Counter16_1_Start
export   _Counter16_1_Start
export    Counter16_1_Stop
export   _Counter16_1_Stop
export    Counter16_1_WritePeriod 
export   _Counter16_1_WritePeriod 
export    Counter16_1_WriteCompareValue
export   _Counter16_1_WriteCompareValue
export    wCounter16_1_ReadCompareValue
export   _wCounter16_1_ReadCompareValue
export    wCounter16_1_ReadCounter
export   _wCounter16_1_ReadCounter

;-----------
;  EQUATES
;-----------
bfCONTROL_REG_START_BIT:   equ   1     ; Control register start bit 
bfINPUT_REG_CLOCK_MASK:    equ   0Fh   ; input register clock mask
bfINPUT_REG_NULL:          equ   00h   ; clear the input register
bfINPUT_REG_PREV_MASK:     equ   30h   ; input register input-previous 
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter16_1_EnableInt
;
;  DESCRIPTION:
;     Enables this counter's interrupt by setting the interrupt enable mask bit
;     associated with this User Module. Remember to call the global interrupt
;     enable function by using the macro: M8C_EnableGInt.
;
;  ARGUMENTS:
;     none.
;
;  RETURNS:
;     none.
;
;  SIDE EFFECTS:
;     none.
;
;  THEORY of OPERATION:  
;     Sets the specific user module interrupt enable mask bit.
;
;-----------------------------------------------------------------------------
 Counter16_1_EnableInt:
_Counter16_1_EnableInt:
   M8C_EnableIntMask Counter16_1_INT_REG, bCounter16_1_INT_MASK
   ret	


;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter16_1_DisableInt
;
;  DESCRIPTION:
;     Disables this counter's interrupt by clearing the interrupt enable mask bit
;     associated with this User Module. 
;
;  ARGUMENTS:
;     none.
;
;  RETURNS:
;     none.
;
;  SIDE EFFECTS:
;     none.
;
;  THEORY of OPERATION:  
;     Clears the specific user module interrupt enable mask bit.
;
;-----------------------------------------------------------------------------
 Counter16_1_DisableInt:
_Counter16_1_DisableInt:
   M8C_DisableIntMask Counter16_1_INT_REG, bCounter16_1_INT_MASK
   ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter16_1_Start
;
;  DESCRIPTION:
;     Sets the start bit in the Control register of this user module.  The
;     counter will begin counting on the next input clock as soon as the 
;     enable input is asserted high.
;
;  ARGUMENTS:
;     none.
;
;  RETURNS:
;     none.
;
;  SIDE EFFECTS:
;     none.
;
;  THEORY of OPERATION:  
;     Set the start bit in the Control register of the LSB block.
;
;-----------------------------------------------------------------------------
 Counter16_1_Start:
_Counter16_1_Start:
   or    REG[Counter16_1_CONTROL_LSB_REG], bfCONTROL_REG_START_BIT
   ret	


;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter16_1_Stop
;
;  DESCRIPTION:
;     Disables counter operation.
;
;  ARGUMENTS:
;     none.
;
;  RETURNS:
;     none.
;
;  SIDE EFFECTS:
;     After this function completes, the Counter register will latch any data
;     written to the Period register.  Writing to the Period register is 
;     performed using the Counter16_1_WritePeriod function.
;
;  THEORY of OPERATION:  
;     Clear the start bit in the Control register of the LSB block.
;
;-----------------------------------------------------------------------------
 Counter16_1_Stop:
_Counter16_1_Stop:
   and   REG[Counter16_1_CONTROL_LSB_REG], ~bfCONTROL_REG_START_BIT
   ret	


;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter16_1_WritePeriod
;
;  DESCRIPTION:
;     Write the period value into the Period register.
;
;  ARGUMENTS:
;     WORD  wPeriodValue - period count - 
;              MSB passed in X register
;              LSB passed in A register
;
;  RETURNS:
;     none.
;
;  SIDE EFFECTS:
;     If the counter user module is stopped, then this value will also be
;     latched into the Counter registers.
;
;  THEORY of OPERATION:  
;     Write data into the Period registers.
;
;-----------------------------------------------------------------------------
 Counter16_1_WritePeriod:
_Counter16_1_WritePeriod:
   tst   REG[Counter16_1_CONTROL_LSB_REG], bfCONTROL_REG_START_BIT
   jnz   .CounterRunning

; Counter is stopped.  Due to chip errata, we have to set the clock low for
; the write to the period register to cause the data to be immediately transferred
; into the Counter.
.CounterStopped:
   push  X
   mov   X, A                                   ; save the period argument
   M8C_SetBank1
   mov   A, REG[Counter16_1_INPUT_LSB_REG]        ; save the context of the clock - input register
   push  A
   and   REG[Counter16_1_INPUT_LSB_REG], F0h      ; set the clock signal low
   and   REG[Counter16_1_INPUT_MSB_REG], F0h      
   M8C_SetBank0
   mov   A, X                                      
   mov   REG[Counter16_1_PERIOD_LSB_REG], A           ; set the period register with the new period
   pop   A
   pop   X
   push  A
   mov   A, X
   mov   REG[Counter16_1_PERIOD_MSB_REG], A               
   pop   A
   M8C_SetBank1
   mov   REG[Counter16_1_INPUT_LSB_REG], A        ; restore the clock
   and   A, 0Fh                                 ; High nibble is always 0x3 for higher order bits 
   or    A, 30h
   mov   REG[Counter16_1_INPUT_MSB_REG], A 
   M8C_SetBank0
   ret

; Counter is running - write the period into the period register.
; Upon Terminal Count this value will get loaded into the counter.
.CounterRunning:
   mov   REG[Counter16_1_PERIOD_LSB_REG], A
   mov   A, X
   mov   REG[Counter16_1_PERIOD_MSB_REG], A
   ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter16_1_WriteCompareValue
;
;  DESCRIPTION:
;     Writes compare value into the CompareValue register.
;
;  ARGUMENTS:
;     WORD  wCompareValue - compare value count - 
;              MSB passed in X register
;              LSB passed in A register
;
;  RETURNS:
;     none.
;
;  SIDE EFFECTS:
;     none.
;
;  THEORY of OPERATION:  
;     Write data into the CompareValue registers.
;
;-----------------------------------------------------------------------------
 Counter16_1_WriteCompareValue:
_Counter16_1_WriteCompareValue:
   mov   REG[Counter16_1_COMPARE_LSB_REG], A 
   mov   A, X
   mov   REG[Counter16_1_COMPARE_MSB_REG], A
   ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: wCounter16_1_ReadCompareValue
;
;  DESCRIPTION:
;     Reads the CompareValue registers.
;
;  ARGUMENTS:
;     none.
;
;  RETURNS:
;     WORD  wCompareValue - value read from CompareValue register - 
;           MSB returned in X register
;           LSB returned in A register           
;
;  SIDE EFFECTS:
;     none.
;
;  THEORY of OPERATION:  
;     Read the CompareValue register and return value in A.
;
;-----------------------------------------------------------------------------
 wCounter16_1_ReadCompareValue:
_wCounter16_1_ReadCompareValue:
   mov   A, REG[Counter16_1_COMPARE_MSB_REG]
   mov   X, A
   mov   A, REG[Counter16_1_COMPARE_LSB_REG]
   ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: wCounter16_1_ReadCounter
;
;  DESCRIPTION:
;     Reads the count in the Counter register.
;
;  ARGUMENTS:
;     none.
;
;  RETURNS:
;     WORD  wCount - current count value in Count register.
;        MSB - returned in X register
;        LSB - returned in A register
;
;  SIDE EFFECTS:
;     Reading the Counter register may cause the Counter register to miss
;     one or more counts due to the fact that the clock is stopped while
;     the Counter register is read.  The preferred method is to use the 
;     interrupt feature to determine when the counter has arrived at a 
;     specified value.
;
;  THEORY of OPERATION:  
;     Reading the Counter register causes its value to be latched into the 
;     CompareValue register.  Care must be taken to stop the clock and save 
;     the CompareValue register's contents before reading the counter.
;
;-----------------------------------------------------------------------------
 wCounter16_1_ReadCounter:
_wCounter16_1_ReadCounter:

   bOrigClockSetting:      EQU   0
   bOrigCompareValue:      EQU   1
   wCounter:               EQU   2     ; must be at top of stack frame
   STACK_FRAME_SIZE:       EQU   4     ; stack frame is 4 bytes

   ; create a stack frame
   mov   X, SP
   ADD   SP, STACK_FRAME_SIZE

   ; save the input register clock setting
   M8C_SetBank1
   mov   A, REG[Counter16_1_INPUT_LSB_REG]
   mov   [X+bOrigClockSetting], A
   ; disable the clock
   mov   REG[Counter16_1_INPUT_LSB_REG], bfINPUT_REG_NULL
   mov   REG[Counter16_1_INPUT_MSB_REG], bfINPUT_REG_PREV_MASK
   M8C_SetBank0

   ;--------------------------
   ;  Process the MSB byte 
   ;--------------------------
   ; save the CompareValue register value
   mov   A, REG[Counter16_1_COMPARE_MSB_REG]    
   mov   [X+bOrigCompareValue], A
   ; Read the counter. This latches the counter data into
   ; the CompareValue register.  This may cause an interrupt.
   mov   A, REG[Counter16_1_COUNTER_MSB_REG]    
   ; Read the CompareValue register, which contains the counter value
   mov   A, REG[Counter16_1_COMPARE_MSB_REG]    
   mov   [X+wCounter], A

   ; Restore the CompareValue register
   mov   A, [X+bOrigCompareValue]
   mov   REG[Counter16_1_COMPARE_MSB_REG], A

   ;--------------------------
   ;  Process the LSB byte 
   ;--------------------------
   ; save the CompareValue register value
   mov   A, REG[Counter16_1_COMPARE_LSB_REG]    
   mov   [X+bOrigCompareValue], A
   ; Read the counter. This latches the counter data into
   ; the CompareValue register.  This may cause an interrupt.
   mov   A, REG[Counter16_1_COUNTER_LSB_REG]    
   ; Read the CompareValue register, which contains the counter value
   mov   A, REG[Counter16_1_COMPARE_LSB_REG]    
   mov   [X+wCounter+1], A

   ; Restore the CompareValue register
   mov   A, [X+bOrigCompareValue]
   mov   REG[Counter16_1_COMPARE_LSB_REG], A

   ; restore the input register clock setting
   M8C_SetBank1
   mov   A, [X+bOrigClockSetting]
   and   A, bfINPUT_REG_CLOCK_MASK
   or    A, bfINPUT_REG_PREV_MASK
   mov   REG[Counter16_1_INPUT_MSB_REG], A
   mov   A, [X+bOrigClockSetting]
   mov   REG[Counter16_1_INPUT_LSB_REG], A
   M8C_SetBank0

   ; Setup the return value in X and A
   mov   A, [X+wCounter]   ; get the MSB
   push  A
   mov   A, [X+wCounter+1] ; LSB in A
   pop   X

   ; Delete the stack frame
   ADD   SP, -(STACK_FRAME_SIZE)

   ret

; end of counter16 API code


