;-----------------------------------------------------------------------------
; Assembly main line
;-----------------------------------------------------------------------------
include "m8c.inc"				;include m8c specific declarations and macros

export _main			
export flags, ticker, period

area	bss	(ram)				;inform assembler of variables to follow
flags:   	blk  1				;contains general purpose flags
ticker:		blk	1				;general purpose "ticker" - decremented at 2Khz
period:		blkw 1				;define period for 16-bit timer
						
area text (ROM, REL)	

_main:
	or 		reg[INT_MSK0],20h
	M8C_EnableGInt
	mov		[period],0			;initialize high byte of period
	mov		[period+1],120		;initialize low byte of period		
	mov		[flags],0
	call	Counter16_1_Start
	call	Counter8_1_Start
	call	Counter8_3_Start		;Turn on Ticker
	call	Counter8_3_EnableInt
	mov		a,3
	call	BPF2_1_Start
	mov		a,3
	call	BPF2_2_Start
	mov		a,3
	call	PGA_1_Start				;Turn on buffer
	mov		reg[PRT1DR],5			;turn on pullups
loop:
	cmp		[flags],1				;look for user input
	jnz		loop
	call	service
	mov		[flags],0	
	jmp		loop
    ret
    
service:
	mov		[ticker],1
debounce:	
	cmp		[ticker],0
	jnz		debounce
	mov		a,reg[PRT1DR]		;test to see if switch is still low
	and		a,1
	jnz		exit
	mov		a,reg[PRT1DR]		;test to see if rotation was CW or CCW
	and		a,4	
	jnz		increment			;if CCW, then increment clock period
decrement:	
	mov		a,[period]			;otherwise, motion is CW so decrement period
	jnz		end_under_test
	mov		a,[period+1]
	cmp		a,5
	jz		exit				;don't decrement if min value is in register
end_under_test:
	dec		[period+1]			
	jnc		done
	dec		[period]
	jmp		done
increment:
	mov		a,[period]
	cmp		a,255
	jnz		end_over_test
	mov		a,[period+1]
	cmp		a,255
	jz		exit				;don't increment if max value reached
end_over_test:
	inc		[period+1]
	jnc		done
	inc		[period]
done:		
	mov		a,[period+1]
	mov		x,[period]
	call	Counter16_1_WritePeriod
exit:	
	ret